%% GLM model for LGN data
function [xfinal] = GLM_LGN
load LGN_FFdata.mat
% choose resolution
r = 2;
spiketimes = FFspks;
stimulus = repelem(FFstim,r);
time_step = DTstim/r;
% define time window
stimuli_length = round(15*r)/2;
spike_length = round(10*r/2);

M = length(stimulus)+ 1 - stimuli_length;

% make a vector of spike times
number_spikes = zeros(1,length(stimulus));
sp_time = floor(spiketimes/time_step);
a = unique(sp_time)';
out = [a,histc(sp_time(:),a)];
for i = 1: length(out)
   number_spikes(out(i,1)) = out(i,2); 
end

% make stimuli preceeding a time moment
stimuli = zeros(stimuli_length,M);
for i = length(stimulus):-1:stimuli_length
    stimuli(:,i-stimuli_length+1) = stimulus(i-stimuli_length+1:i);  
   
end
% make spike stimuli preceeding a time moment
spike_stimuli = zeros(spike_length,M);
for i = length(stimulus):-1:stimuli_length
    spike_stimuli(:,i-stimuli_length+1) = (number_spikes(i-spike_length:i-1));  
   
end
% initial guess
x0 = (ones(1,spike_length+1+stimuli_length)+randn(1,spike_length+1+stimuli_length)/10)*0.1;

xfinal = example(x0,number_spikes,stimuli,spike_stimuli);
figure; plot(xfinal(2:stimuli_length+1))
title ('Optimal linear filter')
xlabel ('time step')
figure; plot(xfinal(stimuli_length+2:end))
title ('History filter (low resolution)')
xlabel ('time step')

% lambda procedure. 1000 is optimal for dt/8 resolution.
% since it is for high resolution, it takes time.
flag = 0;
if flag ==1
lambda = [0,50,100,150,250,500,1000,2000,4000,5000,7000];
FL = lambda_choice(lambda);
end
end 

function xfinal = example(x,spikes,stimuli,spike_stimuli)
options = optimoptions('fminunc','Algorithm','quasi-newton','TolFun',1e-8,'GradObj','on','DerivativeCheck','off','Diagnostics','on','Display','iter');
fun  = @(x)rwg(x,spikes,stimuli,spike_stimuli);
[xfinal,fval,exitflag,output,grad] = fminunc(fun,x,options);
end




function [LL,g] = rwg(x,spikes,stimuli,spike_stimuli)
% choose time step and stimuli length, define resolution
r = 2;
number_spikes = spikes';
stimuli_length = round(15*r/2);
spike_length = round(10*r/2);
s = size(stimuli);
M = s(2);

%%
lam = 500;
LL1 = zeros(M,1);
LL2 = zeros(M,1);
LL3 = zeros(1,stimuli_length);
% Loglikelihood
for i = 1:M
 LL1(i) = number_spikes(i+stimuli_length-1,1)*(x(2:stimuli_length+1)*stimuli(:,i)+ x(stimuli_length+2:end)*spike_stimuli(:,i)+x(1));
 LL2(i) = exp(x(2:stimuli_length+1)*stimuli(:,i)+x(stimuli_length+2:end)*spike_stimuli(:,i)+x(1));

end
% Regularization term
for i = 2:stimuli_length
    LL3(i) = (x(1+i)-x(i))^2;
end

LL = -sum(LL1) + sum(LL2)+ lam*sum(LL3);


if nargout > 1 % gradient
LL11 = zeros(stimuli_length,M);
LL12 = zeros(stimuli_length,M);
LL13 = zeros(1,stimuli_length);
LL21 = zeros(spike_length,M);
LL222 = zeros(spike_length,M);
LL22 = zeros(M,1);
for ii = 1:M
for t = 1:stimuli_length
  LL11(t,ii) = number_spikes(ii+stimuli_length-1,1)*(stimuli(t,ii)) ;
  LL12(t,ii) = stimuli(t,ii)*exp(x(2:stimuli_length+1)*stimuli(:,ii)+x(stimuli_length+2:end)*spike_stimuli(:,ii)+x(1));
  
end

end
for t = 2:stimuli_length-1
    LL13(t) = 2*lam*(x(1+t) - x(t)) - 2*lam*(x(2+t)-x(1+t));
end
LL13(1) = -2*lam*(x(3)-x(2));
LL13(end) = 2*lam*(x(1+stimuli_length)-x(stimuli_length));


for ii = 1:M
     for s = 1:spike_length

         LL21(s,ii) = number_spikes(ii+stimuli_length-1,1)*(spike_stimuli(s,ii)) ;
         LL222(s,ii) = spike_stimuli(s,ii)*exp(x(2:stimuli_length+1)*stimuli(:,ii)+x(stimuli_length+2:end)*spike_stimuli(:,ii)+x(1));
    
     end
 end

for it = 1:M
        LL22(it,1) = exp(x(2:stimuli_length+1)*stimuli(:,it)+x(stimuli_length+2:end)*spike_stimuli(:,it)+x(1));


end
% gradient
g = [-sum(number_spikes)+sum(LL22),(-sum(LL11,2)+sum(LL12,2))'+LL13,(-sum(LL21,2)+sum(LL222,2))'];
   

end



end



%% routines for lambda estimation : runs optimization for a few times(around 11).Takes some time 
function [FL] = lambda_choice(lambda)
load LGN_FFdata.mat
r = 8;
c = 0.5;
k = 1;
spiketimes = FFspks;
stimulus_p = repelem(FFstim,r);
T = round(length(stimulus_p)*c);
stimulus1 = stimulus_p(1:T);
stimulus2 = stimulus_p(T+1:end);
time_step = DTstim/r;
stimuli_length = round(12*r*c);
M1 = length(stimulus1)+ 1 - stimuli_length;
M2 = length(stimulus2)+ 1 - stimuli_length;

% make a vector of spike times
number_spikes = zeros(1,length(stimulus_p));
sp_time = floor(spiketimes/time_step);
a = unique(sp_time)';
out = [a,histc(sp_time(:),a)];
for i = 1: length(out)
   number_spikes(out(i,1)) = out(i,2); 
end
stimuli1 = zeros(stimuli_length,M1);
for i = length(stimulus1):-1:stimuli_length
    stimuli1(:,i-stimuli_length+1) = stimulus1(i-stimuli_length+1:i);  
end
stimuli2 = zeros(stimuli_length,M2);
for i = length(stimulus2):-1:stimuli_length
    stimuli2(:,i-stimuli_length+1) = stimulus2(i-stimuli_length+1:i);    
end
x0 = (ones(1,1+stimuli_length)+randn(1,1+stimuli_length)/10)*0.1;
number_spikes1 = number_spikes(1:T);
number_spikes2 = number_spikes(T+1:end);
LL = zeros(1,11);
for ii = 1:11
lam = lambda(ii);
xfinal = exampleL(x0,number_spikes1,stimuli1,lam);
x = xfinal;
num_spikes = number_spikes2';
LL1 = zeros(M2,1);
LL2 = zeros(M2,1);
for i = 1:M2
LL1(i) = num_spikes(i+stimuli_length-1,1)*(x(2:stimuli_length+1)*stimuli2(:,i)+x(1));
LL2(i) = exp(x(2:stimuli_length+1)*stimuli2(:,i)+ x(1));
end
LL(ii) = -sum(LL1) + sum(LL2);
end
stimulus1 = stimulus_p(T+1:end);
stimulus2 = stimulus_p(1:T);
M1 = length(stimulus1)+ 1 - stimuli_length;
M2 = length(stimulus2)+ 1 - stimuli_length;
stimuli1 = zeros(stimuli_length,M1);
for i = length(stimulus1):-1:stimuli_length
    stimuli1(:,i-stimuli_length+1) = stimulus1(i-stimuli_length+1:i);    
end
stimuli2 = zeros(stimuli_length,M2);
for i = length(stimulus2):-1:stimuli_length
    stimuli2(:,i-stimuli_length+1) = stimulus2(i-stimuli_length+1:i);     
end
x0 = (ones(1,1+stimuli_length)+randn(1,1+stimuli_length)/10)*0.1;
number_spikes1 = number_spikes(T+1:end);
number_spikes2 = number_spikes(1:T);
LL22 = zeros(1,11);
XX22 = zeros(12,length(x0));
for ii = 1:11
lam = lambda(ii);
xfinal = exampleL(x0,number_spikes1,stimuli1,lam);
XX22(i,:) = xfinal;
x = xfinal;
num_spikes = number_spikes2';
LL1 = zeros(M2,1);
LL2 = zeros(M2,1);
for i = 1:M2
LL1(i) = num_spikes(i+stimuli_length-1,1)*(x(2:stimuli_length+1)*stimuli2(:,i)+x(1));
LL2(i) = exp(x(2:stimuli_length+1)*stimuli2(:,i)+ x(1));
end
LL22(ii) = -sum(LL1) + sum(LL2);
end
FL = (LL+LL22)*c;
figure;plot(FL)
title ('Negative LL for different lambda')
xlabel ('lambda values')
figure;plot(LL)
figure;plot(LL22)
end 
function xfinal = exampleL(x,spikes,stimuli,lam)
options = optimoptions('fminunc','Algorithm','quasi-newton','TolFun',1e-8,'GradObj','on','DerivativeCheck','off','Diagnostics','on','Display','iter');
fun  = @(x)rwgL(x,spikes,stimuli,lam);
[xfinal,fval,exitflag,output,grad] = fminunc(fun,x,options);
end
function [LL,g] = rwgL(x,spikes,stimuli,lam)
r = 8;
number_spikes = spikes';
stimuli_length = round(12*r/2);
s = size(stimuli);
M = s(2);
%%
LL1 = zeros(M,1);
LL2 = zeros(M,1);
LL3 = zeros(1,stimuli_length);
for i = 1:M
LL1(i) = number_spikes(i+stimuli_length-1,1)*(x(2:stimuli_length+1)*stimuli(:,i)+x(1));
LL2(i) = exp(x(2:stimuli_length+1)*stimuli(:,i)+ x(1));
end
for i = 2:stimuli_length
    LL3(i) = (x(1+i)-x(i))^2;
end
LL = -sum(LL1) + sum(LL2)+ lam*sum(LL3);

if nargout > 1 % gradient required
LL11 = zeros(stimuli_length,M);
LL12 = zeros(stimuli_length,M);
LL13 = zeros(1,stimuli_length);
LL22 = zeros(M,1);
for ii = 1:M
for t = 1:stimuli_length
  LL11(t,ii) = number_spikes(ii+stimuli_length-1,1)*(stimuli(t,ii)) ;
  LL12(t,ii) = stimuli(t,ii)*exp(x(2:stimuli_length+1)*stimuli(:,ii)+ x(1));   
end

end
for t = 2:stimuli_length-1
    LL13(t) = 2*lam*(x(1+t) - x(t)) - 2*lam*(x(2+t)-x(1+t));
end
LL13(1) = -2*lam*(x(3)-x(2));
LL13(end) = 2*lam*(x(1+stimuli_length)-x(stimuli_length));

for it = 1:M
     LL22(it,1) = exp(x(2:stimuli_length+1)*stimuli(:,it)+x(1));
end  
g = [-sum(number_spikes)+sum(LL22),(-sum(LL11,2)+sum(LL12,2))'+LL13];
end

end

