function [xfinal] = GLM_validation
% validation for the full GLM with k,h and b terms
% generate artificial data
[spikes,stimulus,opfk,opfh] = simulation();
% choose time window length for the linear filter and for the history filter
r =2;
stimuli_length = round(15*r/2);
spike_length = round(10*r/2);

x0 = (ones(1,spike_length+1+stimuli_length)+randn(1,spike_length+1+stimuli_length)/10)*0.1;

xfinal = example(x0,spikes,stimulus);
figure; plot(opfk)
hold on; plot(xfinal(2:stimuli_length+1),'*')
hold off;
title ('Comparison of used linear filter and optimization result')
xlabel ('time step')
legend ('used filter','optimization result')
figure; plot(opfh)
hold on; plot(xfinal(stimuli_length+2:end),'*')
title ('Comparison of used history filter and optimization result')
xlabel ('time step')
legend ('used filter','optimization result')
end 
% optimization routine
function xfinal = example(x,spikes,stimulus)
options = optimoptions('fminunc','Algorithm','quasi-newton','TolFun',1e-8,'GradObj','on','DerivativeCheck','off','Diagnostics','on','Display','iter');
fun  = @(x)rwg(x,spikes,stimulus);
[xfinal,fval,exitflag,output,grad] = fminunc(fun,x,options);
end
% Log-likelihood and Gradient
function [LL,g] = rwg(x,spikes,stimulus)
r = 2;
number_spikes = spikes;
stimuli_length = round(15*r/2);
spike_length = round(10*r/2);
M = length(stimulus)+ 1 - stimuli_length;

% make a matrix of stimuli preceeding times
stimuli = zeros(M,stimuli_length);

for i = length(stimulus):-1:stimuli_length
    stimuli(i-stimuli_length+1,:) = (stimulus(i-stimuli_length+1:i));  
   
end
% make a matrix of spike stimuli preceeding times

spike_stimuli = zeros(M,spike_length);

for i = length(stimulus):-1:stimuli_length
    spike_stimuli(i-stimuli_length+1,:) = (number_spikes(i-spike_length:i-1));  
   
end

%%

LL1 = zeros(M,1);
LL2 = zeros(M,1);
for i = 1:M
   LL1(i) = number_spikes(i+stimuli_length-1,1)*(x(2:stimuli_length+1)*stimuli(i,:)'+ x(stimuli_length+2:end)*spike_stimuli(i,:)'+x(1));
   LL2(i) = exp(x(2:stimuli_length+1)*stimuli(i,:)'+x(stimuli_length+2:end)*spike_stimuli(i,:)'+x(1));
end
% find negative LL
LL = -sum(LL1) + sum(LL2);

if nargout > 1 % gradient required
LL11 = zeros(M,stimuli_length);
LL12 = zeros(M,stimuli_length);
LL21 = zeros(M,spike_length);
LL222 = zeros(M,spike_length);
LL22 = zeros(M,1);
for t = 1:stimuli_length
for ii = 1:M
 LL11(ii,t) = number_spikes(ii+stimuli_length-1,1)*(stimuli(ii,t)) ;
 LL12(ii,t) = stimuli(ii,t)*exp(x(2:stimuli_length+1)*stimuli(ii,:)'+x(stimuli_length+2:end)*spike_stimuli(ii,:)'+x(1));    
end

end

 for s = 1:spike_length
     for ii = 1:M
         LL21(ii,s) = number_spikes(ii+stimuli_length-1,1)*(spike_stimuli(ii,s)) ;
         LL222(ii,s) = spike_stimuli(ii,s)*exp(x(2:stimuli_length+1)*stimuli(ii,:)'+x(stimuli_length+2:end)*spike_stimuli(ii,:)'+x(1));
    
     end
 end

for it = 1:M
       LL22(it,1) = exp(x(2:stimuli_length+1)*stimuli(it,:)'+x(stimuli_length+2:end)*spike_stimuli(it,:)'+x(1));
end
% find gradient
    g = [-sum(number_spikes)+sum(LL22),-sum(LL11,1)+sum(LL12,1),-sum(LL21,1)+sum(LL222,1)];
end
end

% artificial data generation
function [spikes,stimulus,k,h] = simulation()
% generate white0noise stimulus
stimulus = randn(50000,1);
% define time window size
stimuli_length = 15;
spike_length = 10;
M = length(stimulus)+ 1 - stimuli_length;

% make matrix of stimuli using stimulus data 
% the same matrix without first 15 zeros
stimuli = zeros(M,stimuli_length);

for i = length(stimulus):-1:stimuli_length
    stimuli(i-stimuli_length+1,:) = (stimulus(i-stimuli_length+1:i));  
   
end
stimuli2 = zeros(M+stimuli_length-1,stimuli_length);

for i = M+stimuli_length-1:-1:stimuli_length
    stimuli2(i,:) = (stimulus(i-stimuli_length+1:i));  
   
end
% create a linear filter
k = zeros(1,stimuli_length);
for j = 1:stimuli_length
   %k(1,j) = exp((j-15)*3/15);
   k(1,j) = exp((j-15)*3/15)*cos(2*pi*j*(1/7));

end
% create a history filter
h = zeros(1,spike_length);
for j = 1:spike_length
     h(1,j) = -exp((j-spike_length)*5/15);
end
%  choose parameter b
 b = -0.05;
% make spike stimuli preceeding the first moment of time
spike_stimuli0 = zeros(spike_length,1);

% find generator signal, rate. 
% Generate spike using Pisson distribution spikes
gs = zeros(M+stimuli_length-1,1);
rate = zeros(M+stimuli_length-1,1);
spikes = zeros(M+stimuli_length-1,1);

for i = 1:length(stimulus)
  spike_stimuli = spike_stimuli0;
  g = (k)*stimuli2(i,:)' + h*spike_stimuli+b ;
  gs(i,1) = g;
   if i<stimuli_length 
       rate(i,1) = 0;
       spikes(i,1) = 0;
   else   rate(i,1)= exp(g);
       spikes(i,1) = poissrnd(rate(i,1));
      if i < length(stimulus)
          spike_stimuli0 = spikes(i-spike_length+1:i,1);
      end
   end
end
end