function [xfinal] = GQM_validation
% generate stimulus and number of spikes
[spikes,stimulus,opfk,opfb1,opfb2] = simulation();
% choose time window length
stimuli_length = (15);

M = length(stimulus)+ 1 - stimuli_length;

stimuli = zeros(stimuli_length,M);

for i = length(stimulus):-1:stimuli_length
    stimuli(:,i-stimuli_length+1) = (stimulus(i-stimuli_length+1:i));  
   
end
% number of positive/negative filters
N  = 2;
omega = [-1,-1];
x0 = [opfk+randn(1,stimuli_length)/10,opfb1+randn(1,stimuli_length)/10,opfb2+randn(1,stimuli_length)/10];
% find optimal filters
xfinal = example(x0,spikes,stimuli,N, omega);

figure; plot(opfk)
hold on; plot(xfinal(1:stimuli_length),'*')
hold off;
title ('Comparison of used linear filter and optimization result')
xlabel ('time step')
legend ('used filter','optimization result')
figure; plot(opfb2)
hold on; plot(xfinal(2*stimuli_length+1:end),'*')
hold off;
title ('Comparison of 1st used quadratic filter and optimization result')
xlabel ('time step')
legend ('used filter','optimization result')
figure; plot(opfb1)
hold on; plot(xfinal(stimuli_length+1:2*stimuli_length),'*')
title ('Comparison of 2nd used  quadratic filter and optimization result')
xlabel ('time step')
legend ('used filter','optimization result')
end

% optimization routine
function xfinal = example(x,spikes,stimuli,N, omega)
options = optimoptions('fminunc','Algorithm','quasi-newton','TolFun',1e-8,'GradObj','on','DerivativeCheck','off','Diagnostics','on','Display','iter');
fun  = @(x)rwg(x,spikes,stimuli,N, omega);
[xfinal,fval,exitflag,output,grad] = fminunc(fun,x,options);
end


function [LL,g] = rwg(x,spikes,stimuli,N, omega)
number_spikes = spikes;
% define time window
stimuli_length = 15;
s = size(stimuli);
M = s(2);
%% 
% find Negative LL
LL1 = zeros(M,1);
LL2 = zeros(M,1);
for i = 1:M
    q = zeros(1,N);
    for j = 1:N
        q(j) = omega(j)*(x(stimuli_length*j+1:stimuli_length*j+stimuli_length)*stimuli(:,i))^2;
    end
    Q = sum(q);
    LL1(i) = number_spikes(i+stimuli_length-1,1)*(x(1:stimuli_length)*stimuli(:,i)+Q );
    LL2(i) = exp(x(1:stimuli_length)*stimuli(:,i)+ Q );
end
LL = -sum(LL1) + sum(LL2);

% gradient
if nargout > 1 % gradient required
LL11 = zeros(stimuli_length,M);
LL12 = zeros(stimuli_length,M);
LL21 = zeros(stimuli_length*N,M);
LL22 = zeros(stimuli_length*N,M);

for ii = 1:M
    q = zeros(1,N);
    for j = 1:N
        q(j) = omega(j)*(x(stimuli_length*j+1:stimuli_length*j+stimuli_length)*stimuli(:,ii))^2;
    end
    Q = sum(q);
    
    for t = 1:stimuli_length
    LL11(t,ii) = number_spikes(ii+stimuli_length-1,1)*(stimuli(t,ii)) ;
    LL12(t,ii) = stimuli(t,ii)*exp(x(1:stimuli_length)*stimuli(:,ii) + Q);
    end
    for tt = 1:stimuli_length*N
              j = ceil(tt/stimuli_length);
                if tt>=1 && tt<=stimuli_length
                                    ttt = tt;
                                      LL21(tt,ii ) = 2*number_spikes(ii+stimuli_length-1,1)*omega(j)*stimuli(ttt,ii)*x(stimuli_length+1:2*stimuli_length)*stimuli(:,ii);
                                      LL22(tt,ii ) = 2*omega(j)*stimuli(ttt,ii)*x(stimuli_length+1:2*stimuli_length)*stimuli(:,ii)*exp(x(1:stimuli_length)*stimuli(:,ii) +Q);
                else
                                     ttt = tt-stimuli_length*(j-1);
                                      LL21(tt,ii ) = 2*number_spikes(ii+stimuli_length-1,1)*stimuli(ttt,ii)*omega(j)*x(stimuli_length*j+1:stimuli_length*j+stimuli_length)*stimuli(:,ii);
                                      LL22(tt,ii ) = 2*omega(j)*stimuli(ttt,ii)*x(stimuli_length*j+1:stimuli_length*j+stimuli_length)*stimuli(:,ii)*exp(x(1:stimuli_length)*stimuli(:,ii) +Q);
                end
    end
end
% gradient
   g = [(-sum(LL11,2)+sum(LL12,2))',(-sum(LL21,2)+sum(LL22,2))'];
end
end

% generate artificial data 
function [spikes,stimulus,k,b1,b2] = simulation()
% white noise stimulus
stimulus = randn(50000,1);
stimuli_length = 15;
M = length(stimulus)+ 1 - stimuli_length;
% make a matrix of stimuli preceeding times
stimuli2 = zeros(M+stimuli_length-1,stimuli_length);

for i = M+stimuli_length-1:-1:stimuli_length
    stimuli2(i,:) = (stimulus(i-stimuli_length+1:i));  
   
end
% linear filter
k = zeros(1,stimuli_length);
for j = 1:stimuli_length
   k(1,j) = exp((j-15)*3/15)*cos(2*pi*j*(1/7));
end
% quadratic filters
b1 = zeros(1,stimuli_length);
for j = 1:stimuli_length
     b1(1,j) = exp((j-15)*2/15)*sin(pi*j*(1/7)-2);
end
b2 = zeros(1,stimuli_length);
for j = 1:stimuli_length
     b2(1,j) = exp((j-15)*2/15)*sin(pi*j*(1/7)+6);
end
% find generator signal, rate.
% Generate spikes using Poisson distribution 
gs = zeros(M+stimuli_length-1,1);
rate = zeros(M+stimuli_length-1,1);
spikes = zeros(M+stimuli_length-1,1);

for i = 1:length(stimulus)
  g = (k)*stimuli2(i,:)'  - (b1*stimuli2(i,:)')^2 - (b2*stimuli2(i,:)')^2;
  gs(i,1) = g;
  rate(i,1) = exp(g);
  spikes(i,1) = poissrnd(rate(i,1));
end

end