% upload real data and use LNP model with STA filter
load LGN_FFdata.mat

spiketimes = FFspks;
stimulus = FFstim;
time_step = DTstim;


%% give general names to variables

% use low resolution in this case and work with 120 sec stimulus
stimulus = repelem(stimulus,1);
time_step = time_step;

% set up N, M and assume stimuli length
stimuli_length = 15;
N = length(spiketimes);
M = length(stimulus)+ 1 - stimuli_length;

% find how many spikes were at a particular moment of time
n = zeros(1,length(stimulus));
sp_time = round(spiketimes/time_step);
a = unique(sp_time)';
out = [a,histc(sp_time(:),a)];
for i = 1: length(out)
   n(out(i,1)) = out(i,2); 
end
n(1:stimuli_length-1) = 0;
% make matrix of stimuli
stimuli = zeros(M,stimuli_length);

for i = 1: M-1
    stimuli(i,:) = stimulus(i:i+stimuli_length-1);
   
end

%% STA filter
% find average stimuli
s_av = sum(stimuli,1)/M;
%find difference between stimuli and average stimuli
sta = zeros(M,stimuli_length);
for t = 1:M
    sta(t,:) = stimuli(t,:) - s_av;
end

% multiply the respective stimuli part by the number of spikes
sta_pre = zeros(length(n),stimuli_length);
for i = length(n):-1:stimuli_length
    sta_pre(i,:) = n(i)*sta(i-stimuli_length+1,:);
end

%%STA itself
sta_f = (1/N)*sum(sta_pre,1);

figure; plot(sta_f)
title ('STA filter at the low resolution')
xlabel ('time step  (one time step corresponds to 0.0083 sec)')

%% generator signal for 120 sec stimulus
g_with_time = zeros(M+stimuli_length-1,2);
stimuli2 = zeros(M+stimuli_length-1,stimuli_length);

for i = M+stimuli_length-1:-1:stimuli_length
    stimuli2(i,:) = (stimulus(i-stimuli_length+1:i));  
   
end
% choose number of bins for the histogram
w = 20;
fn =figure;
for i = 1:M
   g = (sta_f)*stimuli2(i,:)';
   g_with_time(i,1) = g;
   g_with_time(i,2) = time_step*i;
end
figure;
h = histogram(g_with_time(:,1),w);
title ('histogram for generator signal, which was found for 120 sec stimulus')
xlabel ('generator signal values')
counts = h.Values;
%bin_size = h.BinWidth;
%lim = h.BinLimits;
%intervals = lim(1,1):bin_size:lim(1,2);
%intervals = [intervals,lim(1,2)];
bin_size = (abs(min(g_with_time(:,1)))+max(g_with_time(:,1)))/w;
intervals = min(g_with_time(:,1)):bin_size:max(g_with_time(:,1));
Spike_per_bin = zeros(1,w+1);
%% estimate non-linearty F for 120 sec stimulus
for i=1:length(intervals)-1
    a = intervals(1,i);
    b = intervals (1,i+1);
    k = 0;
    sp = 0;
    Ind_av = find(g_with_time(:,1)>=a & g_with_time(:,1)<b);
    for j = 1:length(Ind_av)
        k = [k;find(out(:,1)==Ind_av(j))];
    end
    for j = 1:length(k)
        if k(j)~=0
            sp = sp + out(k(j),2);
        end
    end
    if length(Ind_av)>0
        Spike_per_bin(1,i+1) = sp/length(Ind_av);
    end
end
figure;
plot(intervals(1,1:length(intervals)),Spike_per_bin(1,1:length(intervals)))
title ('Non-linearity F for 120 sec stimulus ')
xlabel ('generator signal values')
ylabel ('spike rate (spike per bin)')


%% take new data - another stimulus : repeated stimulus and corresponding spike times
MM = length(FFstimR)+ 1 - stimuli_length;
g_with_time3 = zeros(MM+stimuli_length-1,2);
stimuli3 = zeros(MM+stimuli_length-1,stimuli_length);

for i = MM+stimuli_length-1:-1:stimuli_length
    stimuli3(i,:) = (FFstimR(i-stimuli_length+1:i));  
   
end
% choose number of bins for the histogram
w = 20;
% find generator signal using sta filter found for the other stimulus
for i = 1:MM
   g = (sta_f)*stimuli3(i,:)';
   g_with_time3(i,1) = g;
   g_with_time3(i,2) = time_step*i;
end

% using found above non-linearity F find the spike rate
rate = 0;
for i =1:1200
    
    for j = 1:w
        if g_with_time3(i,1)>=intervals(j) && g_with_time3(i,1)<intervals(j)+bin_size/2
        rate = [rate,Spike_per_bin(j)];
        end
        if g_with_time3(i,1)>=intervals(j)+bin_size/2 && g_with_time3(i,1)<intervals(j+1)
        rate = [rate,Spike_per_bin(j+1)];
        end
    end
    
end
rate = rate(2:end);

ind = find(FFspksR== -1);
ind = [1,ind];
n2 = zeros(64,length(FFstimR));

for k = 1:length(ind)-1
sp_time2 = round(FFspksR(1*ind(k)+1:ind(k+1)-1)/time_step);
a2 = unique(sp_time2)';
out2 = [a2,histc(sp_time2(:),a2)];
for i = 1: length(out2)
   n2(k,out2(i,1)) = out2(i,2); 
end
n2(k,1:stimuli_length-1) = 0;
end
nn = sum(n2,1)/64;
% find the measured spike rate
newnn = 0;
newrate = 0;
for i = 1:10:length(nn)-1
    newnn = [newnn,sum(nn(i:i+1))];
    newrate = [newrate,sum(rate(i:i+1))];
end
figure;
plot(newnn)
hold on
plot(newrate,'r')
title ('Comparison of measured and estimated spike rates')
xlabel ('time step (one time step corresponds to 0.0083 sec)')
ylabel ('spike rate (spike per bin)')
legend ('measured','estimated')
%% R_squared = 1 - (mean squared error) / (variance of observed response)
var_obs = sum(newnn.^2)/121;
mean_sq_error = sum((newnn-newrate).^2)/121;
R_squared_rate = 1 - mean_sq_error/var_obs


% used for plotting
% newnn = newnn(2:end);
% newrate = newrate(2:end);
% figure;plot(time,newnn)
% hold on
% plot(time,newrate,'r')
% time = 0:-time_step:-1;
% vec = V2(:,2);
% time_2 = 0;
