function [xfinal] = NIM_1
load RGCsimdata.mat
% choose resolution and stimuli length
r = 1;
stimulus = repelem(stim,r);
time_step = stim_dt/r;
stimuli_length = 15*r;
M = length(stimulus)+ 1 - stimuli_length;
stimuli = zeros(stimuli_length,M);
for i = length(stimulus):-1:stimuli_length
    stimuli(:,i-stimuli_length+1) = stimulus(i-stimuli_length+1:i);  
   
end

% make a vector of spike times
number_spikes = zeros(1,length(stimulus));
sp_time = round(spiketimes/time_step);
a = unique(sp_time);
out = [a,histc(sp_time(:),a)];
for i = 1: length(out)
   number_spikes(out(i,1)) = out(i,2); 
end

%%

%number of positive and negative filters
N = 2;
omega = [1,1];
x0 = [(ones(1,stimuli_length)-randn(1,stimuli_length))/5,-ones(1,stimuli_length)*0.1];

xfinal = example(x0,number_spikes,stimuli,N,omega);

figure; plot(xfinal(1:stimuli_length));
hold on;plot(xfinal(stimuli_length+1:end))
title ('NIM filters')
xlabel ('time step')
legend ('1st filter','2nd filter')
end 

function xfinal = example(x,spikes,stimuli,N,omega)
options = optimoptions('fminunc','Algorithm','quasi-newton','TolFun',1e-8,'GradObj','on','Diagnostics','on','Display','iter');
fun  = @(x)rwg(x,spikes,stimuli,N,omega);
[xfinal,fval,exitflag,output,grad] = fminunc(fun,x,options);
end


function [LL,g] = rwg(x,spikes,stimuli,N,omega)
number_spikes = spikes';
% resolution
stimuli_length = 15*1;
s = size(stimuli);
M = s(2);

%% log-likelihood
LL1 = zeros(M,1);
LL2 = zeros(M,1);
% choose lambda
lam = 0;
LL3 = zeros(stimuli_length,1);
LL4 = zeros(stimuli_length,1);

for i = 1:M
    q = zeros(1,N);
    for j = 1:N
        q(j) = omega(j)*log(1+exp(x(stimuli_length*(j-1)+1:stimuli_length*(j-1)+stimuli_length)*stimuli(:,i)));
    end
    Q = sum(q);
    LL1(i) = number_spikes(i+stimuli_length-1,1)*Q;
    LL2(i) = exp(Q);
end

for i = 1:stimuli_length-1
    LL3(i) = (x(1+i)-x(i))^2;
end
for i = stimuli_length+1:2*stimuli_length-1
    LL4(i-stimuli_length) = (x(1+i)-x(i))^2;
end
LL5 = [LL3;LL4];
LL = -sum(LL1) + sum(LL2) + lam*sum(LL5);

% gradient
if nargout > 1 % gradient required
LL11 = zeros(stimuli_length*N,M);
LL12 = zeros(stimuli_length*N,M);
LL13 = zeros(stimuli_length,1);
for ttt = 2:stimuli_length-1
    LL13(ttt) = 2*lam*(x(ttt) - x(ttt-1)) - 2*lam*(x(1+ttt)-x(ttt));
end
LL13(1) = -2*lam*(x(2)-x(1));
LL13(end) = 2*lam*(x(stimuli_length)-x(stimuli_length-1));
LL14 = zeros(stimuli_length,1);
for ttt = stimuli_length+2:2*stimuli_length-1
    LL14(ttt-stimuli_length) = 2*lam*(x(ttt) - x(ttt-1)) - 2*lam*(x(1+ttt)-x(ttt));
end
LL14(1) = -2*lam*(x(stimuli_length+2)-x(stimuli_length+1));
LL14(end) = 2*lam*(x(2*stimuli_length)-x(stimuli_length*2-1));

for ii = 1:M
    q = zeros(1,N);
    for j = 1:N
        q(j) = omega(j)*log(1+exp(x(stimuli_length*(j-1)+1:stimuli_length*(j-1)+stimuli_length)*stimuli(:,ii)));
    end
    Q = sum(q);
     
    for n = 1:N
    for t = stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length
    if t>=1 && t<=stimuli_length
    tt = t;
    LL11(t,ii) = number_spikes(ii+stimuli_length-1,1)*omega(n)*(stimuli(tt,ii))*...
        (1/(1+exp(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii))))*(exp(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii)));
    LL12(t,ii) = stimuli(tt,ii)*exp(Q)*omega(n)*...
        (1/(1+exp(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii))))*(exp(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii)));
    else
    tt = t-stimuli_length*(n-1);
    LL11(t,ii) = number_spikes(ii+stimuli_length-1,1)*omega(n)*(stimuli(tt,ii))*...
        (1/(1+exp(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii))))*(exp(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii)));
    LL12(t,ii) = stimuli(tt,ii)*exp(Q)*omega(n)*...
        (1/(1+exp(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii))))*(exp(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii)));
    end
    end
    
    end
    
    

end
LL55 = [LL13;LL14]';
g = (-sum(LL11,2)+sum(LL12,2))'+LL55;
      
end



    




end
