function [xfinal] = NIM_validation
%  generate artificial data
[spikes,stimulus,opfb1,opfb2] = simulation();
% choose resolution
r = 1;
stimuli_length = (15*r);
M = length(stimulus)+ 1 - stimuli_length;
% form matrix of stimuli preceeding times
stimuli = zeros(stimuli_length,M);
for i = length(stimulus):-1:stimuli_length
    stimuli(:,i-stimuli_length+1) = stimulus(i-stimuli_length+1:i);  
   
end
% number of positive/negative filters
N  = 2;
omega = [1,1];
x0 = [(opfb1+randn(1,stimuli_length))/10,(opfb2+randn(1,stimuli_length))/10];
% find optimal filters for the given data
xfinal = example(x0,spikes,stimuli,N, omega);
figure; plot(opfb1)
hold on; plot(xfinal(1:stimuli_length),'*');
hold off;
title ('Comparison of 1st used filter and optimization result')
xlabel ('time step')
legend ('used filter','optimization result')
figure; plot(opfb2)
hold on;plot(xfinal(stimuli_length+1:end),'*')
title ('Comparison of 2nd used filter and optimization result')
xlabel ('time step')
legend ('used filter','optimization result')
end 
% optimization routine
function xfinal = example(x,spikes,stimuli,N,omega)
options = optimoptions('fminunc','Algorithm','quasi-newton','TolFun',1e-8,'GradObj','on','Diagnostics','on','Display','iter');
fun  = @(x)rwg(x,spikes,stimuli,N,omega);
[xfinal,fval,exitflag,output,grad] = fminunc(fun,x,options);
end
% Negative LL and gradient functions
function [LL,g] = rwg(x,spikes,stimuli,N,omega)
number_spikes = spikes;
% chose resolution
stimuli_length = 15*1;
s = size(stimuli);
M = s(2);

%%
LL1 = zeros(M,1);
LL2 = zeros(M,1);
% choose lambda
lam = 0;
LL3 = zeros(stimuli_length,1);
LL4 = zeros(stimuli_length,1);

for i = 1:M
    q = zeros(1,N);
    for j = 1:N
        q(j) = omega(j)*ff(x(stimuli_length*(j-1)+1:stimuli_length*(j-1)+stimuli_length)*stimuli(:,i));
    end
    Q = sum(q);
    LL1(i) = number_spikes(i+stimuli_length-1,1)*Q;
    LL2(i) = exp(Q);
end

for i = 1:stimuli_length-1
    LL3(i) = (x(1+i)-x(i))^2;
end
for i = stimuli_length+1:2*stimuli_length-1
    LL4(i-stimuli_length) = (x(1+i)-x(i))^2;
end
LL5 = [LL3;LL4];
% find Negative LL
LL = -sum(LL1) + sum(LL2) + lam*sum(LL5);

if nargout > 1 % gradient formula
LL11 = zeros(stimuli_length*N,M);
LL13 = zeros(stimuli_length,1);
for ttt = 2:stimuli_length-1
    LL13(ttt) = 2*lam*(x(ttt) - x(ttt-1)) - 2*lam*(x(1+ttt)-x(ttt));
end
LL13(1) = -2*lam*(x(2)-x(1));
LL13(end) = 2*lam*(x(stimuli_length)-x(stimuli_length-1));
LL14 = zeros(stimuli_length,1);
for ttt = stimuli_length+2:2*stimuli_length-1
    LL14(ttt-stimuli_length) = 2*lam*(x(ttt) - x(ttt-1)) - 2*lam*(x(1+ttt)-x(ttt));
end
LL14(1) = -2*lam*(x(stimuli_length+2)-x(stimuli_length+1));
LL14(end) = 2*lam*(x(2*stimuli_length)-x(stimuli_length*2-1));

for ii = 1:M
    q = zeros(1,N);
    for j = 1:N
        q(j) = omega(j)*ff(x(stimuli_length*(j-1)+1:stimuli_length*(j-1)+stimuli_length)*stimuli(:,ii));
    end
    Q = sum(q);
     
    for n = 1:N
    for t = stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length
    if t>=1 && t<=stimuli_length
    tt = t;
    LL11(t,ii) = (number_spikes(ii+stimuli_length-1,1)-exp(Q))*omega(n)*(stimuli(tt,ii))*fff(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii)) ;
    else
    tt = t-stimuli_length*(n-1);
    LL11(t,ii) = (number_spikes(ii+stimuli_length-1,1)-exp(Q))*omega(n)*(stimuli(tt,ii))*fff(x(stimuli_length*(n-1)+1:stimuli_length*(n-1)+stimuli_length)*stimuli(:,ii)) ;
    end
    end
    end
    
    

end

LL55 = [LL13;LL14]';
g = (-sum(LL11,2))'+LL55;
      
end
end
function a = ff(x)
if x<=0
    a = 0;
else a = x;
end
end
function a = fff(x)
if x<=0
    a = 0;
else a = 1;
end
end

function [spikes,stimulus,b1,b2] = simulation()
r =1 ;
stimulus = randn(50000*1,1);
stimuli_length = 15*r;
M = length(stimulus)+ 1 - stimuli_length;

stimuli2 = zeros(M+stimuli_length-1,stimuli_length);

for i = M+stimuli_length-1:-1:stimuli_length
    stimuli2(i,:) = (stimulus(i-stimuli_length+1:i));  
   
end
% generate filters
b1 = zeros(1,stimuli_length);
for j = 1:stimuli_length
     b1(1,j) = exp((j-stimuli_length)*2/stimuli_length)*sin(pi*j*(1/7)*(1/r)-2);
end
b2 = zeros(1,stimuli_length);
for j = 1:stimuli_length
     b2(1,j) = exp((j-stimuli_length)*2/stimuli_length)*sin(pi*j*(1/7)*(1/r)+6);
end
% find generator signal, rate.
% Generate spikes using Poisson random number generator
gs = zeros(M+stimuli_length-1,1);
rate = zeros(M+stimuli_length-1,1);
spikes = zeros(M+stimuli_length-1,1);

for i = 1:length(stimulus)
  g = ff(b1*stimuli2(i,:)') + ff(b2*stimuli2(i,:)');
  gs(i,1) = g;
  rate(i,1) = exp(g);
  spikes(i,1) = poissrnd(rate(i,1));  
end

end