% upload synthetic data set.
load RGCsimdata.mat

% choose time step and stimuli length - low resolution is used. 
% in order to switch to high resolution and get smoother graphs, which 
% look like in Prof. Butts paper - change in lines 8,9,10  "1" to "8". In
% this case time step becomes equal to 0.0667/8.
stimulus = repelem(stim,4);
time_step = stim_dt/4;
stimuli_length = 15*4;

N = length(spiketimes);
M = length(stimulus)+ 1 - stimuli_length;

% find how many spikes were at a particular moment of time
number_spikes = zeros(1,length(stimulus));
sp_time = ceil(spiketimes/time_step);
a = unique(sp_time);
out = [a,histc(sp_time(:),a)];
for i = 1: length(out)
   number_spikes(out(i,1)) = out(i,2); 
end

% number_spikes is a matrix of the length of stimulus, which shows how many
% spikes were at time starting from 0 with step time_step. Since we use
% stimuli preceeding the spike, all spikes occured earlier are not taken
% into account
number_spikes(1:stimuli_length-1) = 0;

% make matrix of stimuli using stimulus data 
stimuli = zeros(M,stimuli_length);

for i = length(stimulus):-1:stimuli_length
    stimuli(i-stimuli_length+1,:) = (stimulus(i-stimuli_length+1:i));  
   
end

%% find STA filter
% find an average stimuli
s_av = sum(stimuli,1)/M;

%find difference between stimuli and average stimuli
sta = zeros(M,stimuli_length);
for t = 1:M
    sta(t,:) = stimuli(t,:) - s_av;
end

% multiply the respective stimuli part by the number of spikes
sta_pre = zeros(length(number_spikes),stimuli_length);
for i = length(number_spikes):-1:stimuli_length
    sta_pre(i,:) = number_spikes(i)*sta(i-stimuli_length+1,:);
end

%STA itself for low time resoultion
sta_f = (1/N)*sum(sta_pre,1);
figure; plot(sta_f)
title ('STA filter at the low resolution')
xlabel ('time step  (one time step corresponds to 0.0667 sec)')
time = 0:time_step:1;
figure;plot(time(2:end),sta_f)

%% find STC

% M is number of stimuli, sta_f is sta filter
stimuli2 = zeros(M+stimuli_length-1,stimuli_length);

for i = M+stimuli_length-1:-1:stimuli_length
    stimuli2(i,:) = (stimulus(i-stimuli_length+1:i));  % may be do fliplr
   
end

% find only triggered
tstc2 = 0;
check = find(number_spikes ~=0);
Total = sum(number_spikes);
for t = length(stimuli2):-1:stimuli_length
    tstc2 = tstc2 + number_spikes(t)*(stimuli2(t,:) - sta_f)'*(stimuli2(t,:) - sta_f)  ; 
end
tstc_f2 = (1/(Total-1))*(tstc2);

% find untriggered
unstc2 = 0;
for t = length(stimuli2):-1:stimuli_length
    
    unstc2 = unstc2 +  (stimuli2(t,:) -sta_f)'*(stimuli2(t,:)-sta_f) ; 
    
end
unstc_f2 = (1/(length(stimuli2)-stimuli_length-1))*(unstc2);

stc_f= tstc_f2 - unstc_f2; 

% find eigen values, which are outliers
[V2,D2] = eig(stc_f);
figure; plot(diag(D2),'o')
title('Eigen values')

% concequently V2(:,1) and V2(:,2) are the filters. Plot only the filter
% which was used in the paper.
figure; plot(V2(:,2))
title('STC filter at the low resolution')
xlabel ('time step  (one time step corresponds to 0.0667 sec)')


st = zeros(M,15);
for i =1:15
for t = 1:M
    st(t,i) = stimuli(t,i)*stimuli(t,i);
end
end
st_f = sum(st,1);
sta_ff = sum(sta_pre,1);
% example how the stimulus look like - graph for low resolution case

% time = 0:time_step:10;
% figure;plot(time, stimulus(1:151));axis([0 10 -1.5 1.5]) 
% title('Original stimulus')
% xlabel ('time(sec)')
